@@ -3,8 +3,7 @@
 
 All the matplotlib support code was co-developed with John Hunter,
 matplotlib's author.
-
-$Id: Shell.py 3024 2008-02-07 15:34:42Z darren.dale $"""
+"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
@@ -13,10 +12,6 @@ $Id: Shell.py 3024 2008-02-07 15:34:42Z darren.dale $"""
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>' % Release.authors['Fernando']
-__license__ = Release.license
-
 # Code begins
 # Stdlib imports
 import __builtin__
@@ -297,17 +292,19 @@ if HAS_CTYPES:
         """raises the exception, performs cleanup if needed"""
         if not inspect.isclass(exctype):
             raise TypeError("Only types can be raised (not instances)")
-        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid,
+        # Explicit cast to c_long is necessary for 64-bit support:
+        # See https://bugs.launchpad.net/ipython/+bug/237073
+        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(tid),
                                                          ctypes.py_object(exctype))
         if res == 0:
             raise ValueError("invalid thread id")
         elif res != 1:
-            # """if it returns a number greater than one, you're in trouble, 
-            # and you should call it again with exc=NULL to revert the effect"""
+            # If it returns a number greater than one, you're in trouble, 
+            # and you should call it again with exc=NULL to revert the effect
             ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, 0)
             raise SystemError("PyThreadState_SetAsyncExc failed")
 
-    def sigint_handler (signum,stack_frame):
+    def sigint_handler(signum,stack_frame):
         """Sigint handler for threaded apps.
 
         This is a horrible hack to pass information about SIGINT _without_
@@ -326,7 +323,7 @@ if HAS_CTYPES:
             Term.cout.flush()
 
 else:
-    def sigint_handler (signum,stack_frame):
+    def sigint_handler(signum,stack_frame):
         """Sigint handler for threaded apps.
 
         This is a horrible hack to pass information about SIGINT _without_
@@ -385,7 +382,7 @@ class MTInteractiveShell(InteractiveShell):
 
         Modified version of code.py's runsource(), to handle threading issues.
         See the original for full docstring details."""
-        
+                
         global KBINT
         
         # If Ctrl-C was typed, we reset the flag and return right away
@@ -415,7 +412,7 @@ class MTInteractiveShell(InteractiveShell):
         if (self.worker_ident is None
             or self.worker_ident == thread.get_ident() ):
             InteractiveShell.runcode(self,code)
-            return
+            return False
 
         # Case 3
         # Store code in queue, so the execution thread can handle it.
@@ -776,6 +773,17 @@ class IPShellGTK(IPThread):
                  debug=1,shell_class=MTInteractiveShell):
 
         import gtk
+        ## # Check for set_interactive, coming up in new pygtk.
+        ## # Disable it so that this code works, but notify 
+        ## # the user that he has a better option as well.
+        ## # XXX TODO better support when set_interactive is released
+        ## try:
+        ##     gtk.set_interactive(False)
+        ##     print "Your PyGtk has set_interactive(), so you can use the"
+        ##     print "more stable single-threaded Gtk mode."
+        ##     print "See https://bugs.launchpad.net/ipython/+bug/270856"
+        ## except AttributeError:
+        ##     pass
         
         self.gtk = gtk
         self.gtk_mainloop = hijack_gtk()
@@ -1141,6 +1149,18 @@ class IPShellMatplotlibQt4(IPShellQt4):
 #-----------------------------------------------------------------------------
 # Factory functions to actually start the proper thread-aware shell
 
+def check_gtk(mode):
+    try:
+        import gtk
+    except ImportError:
+        return mode
+    if hasattr(gtk,'set_interactive'):
+        gtk.set_interactive(False)
+        return 'tkthread'
+    else:
+        return mode
+
+
 def _select_shell(argv):
     """Select a shell from the given argv vector.
 
@@ -1210,6 +1230,9 @@ def _select_shell(argv):
             else:
                 # Any other backend, use plain Tk
                 th_mode = 'tkthread'
+
+            # New versions of pygtk don't need the brittle threaded support.
+            th_mode = check_gtk(th_mode)
                 
         return mpl_shell[th_mode]
     else:
@@ -1218,6 +1241,9 @@ def _select_shell(argv):
             th_mode = special_opts.pop()
         except KeyError:
             th_mode = 'tkthread'
+
+        # New versions of pygtk don't need the brittle threaded support.
+        th_mode = check_gtk(th_mode)
         return th_shell[th_mode]
 
 
