@@ -221,7 +221,6 @@ class Demo(object):
           """
         if hasattr(src, "read"):
              # It seems to be a file or a file-like object
-            self.fobj = src
             self.fname = "from a file-like object"
             if title == '':
                 self.title = "from a file-like object"
@@ -229,7 +228,6 @@ class Demo(object):
                 self.title = title
         else:
              # Assume it's a string or something that can be converted to one
-            self.fobj = open(src)
             self.fname = src
             if title == '':
                 (filepath, filename) = os.path.split(src)
@@ -238,6 +236,7 @@ class Demo(object):
                 self.title = title
         self.sys_argv = [src] + shlex.split(arg_str)
         self.auto_all = auto_all
+        self.src = src
         
         # get a few things from ipython.  While it's a bit ugly design-wise,
         # it ensures that things like color scheme and the like are always in
@@ -252,9 +251,22 @@ class Demo(object):
         # load user data and initialize data structures
         self.reload()
 
+    def fload(self):
+        """Load file object."""
+        # read data and parse into blocks
+        if hasattr(self, 'fobj') and self.fobj is not None:
+           self.fobj.close()
+        if hasattr(self.src, "read"):
+             # It seems to be a file or a file-like object
+            self.fobj = self.src
+        else:
+             # Assume it's a string or something that can be converted to one
+            self.fobj = open(self.fname)
+
     def reload(self):
         """Reload source from disk and initialize state."""
-        # read data and parse into blocks
+        self.fload()
+        
         self.src     = self.fobj.read()
         src_b        = [b.strip() for b in self.re_stop.split(self.src) if b]
         self._silent = [bool(self.re_silent.findall(b)) for b in src_b]
@@ -494,14 +506,20 @@ class LineDemo(Demo):
     This class doesn't require any markup at all, and it's meant for simple
     scripts (with no nesting or any kind of indentation) which consist of
     multiple lines of input to be executed, one at a time, as if they had been
-    typed in the interactive prompt."""
+    typed in the interactive prompt.
+
+    Note: the input can not have *any* indentation, which means that only
+    single-lines of input are accepted, not even function definitions are
+    valid."""
     
     def reload(self):
         """Reload source from disk and initialize state."""
         # read data and parse into blocks
-        src_b           = [l for l in self.fobj.readline() if l.strip()]
+        self.fload()
+        lines           = self.fobj.readlines()
+        src_b           = [l for l in lines if l.strip()]
         nblocks         = len(src_b)
-        self.src        = os.linesep.join(self.fobj.readlines())
+        self.src        = ''.join(lines)
         self._silent    = [False]*nblocks
         self._auto      = [True]*nblocks
         self.auto_all   = True
