@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 """Mimic C structs with lots of extra functionality.
-
-$Id: ipstruct.py 1950 2006-11-28 19:15:35Z vivainio $"""
+"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2004 Fernando Perez <fperez@colorado.edu>
@@ -10,10 +9,6 @@ $Id: ipstruct.py 1950 2006-11-28 19:15:35Z vivainio $"""
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>' % Release.authors['Fernando']
-__license__ = Release.license
-
 __all__ = ['Struct']
 
 import types
@@ -55,24 +50,32 @@ class Struct:
     Define a dictionary and initialize both with dict and k=v pairs:
     >>> d={'a':1,'b':2}
     >>> s=Struct(d,hi=10,ho=20)
+
     The return of __repr__ can be used to create a new instance:
     >>> s
-    Struct({'ho': 20, 'b': 2, 'hi': 10, 'a': 1})
+    Struct({'__allownew': True, 'a': 1, 'b': 2, 'hi': 10, 'ho': 20})
+
+    Note: the special '__allownew' key is used for internal purposes.
+    
     __str__ (called by print) shows it's not quite a regular dictionary:
     >>> print s
-    Struct {a: 1, b: 2, hi: 10, ho: 20}
+    Struct({'__allownew': True, 'a': 1, 'b': 2, 'hi': 10, 'ho': 20})
+
     Access by explicitly named key with dot notation:
     >>> s.a
     1
+
     Or like a dictionary:
     >>> s['a']
     1
+
     If you want a variable to hold the key value, only dictionary access works:
     >>> key='hi'
     >>> s.key
     Traceback (most recent call last):
       File "<stdin>", line 1, in ?
     AttributeError: Struct instance has no attribute 'key'
+
     >>> s[key]
     10
 
@@ -80,14 +83,17 @@ class Struct:
     initialization): keys can't be numbers. But numeric keys can be used and
     accessed using the dictionary syntax. Again, an example:
 
-    This doesn't work:
-    >>> s=Struct(4='hi')
+    This doesn't work (prompt changed to avoid confusing the test system):
+    ->> s=Struct(4='hi')
+    Traceback (most recent call last):
+        ...
     SyntaxError: keyword can't be an expression
+
     But this does:
     >>> s=Struct()
     >>> s[4]='hi'
     >>> s
-    Struct({4: 'hi'})
+    Struct({4: 'hi', '__allownew': True})
     >>> s[4]
     'hi'
     """
@@ -152,8 +158,22 @@ class Struct:
         return self.__dict__[key]
 
     def __contains__(self,key):
-        """Allows use of the 'in' operator."""
-        return self.__dict__.has_key(key)
+        """Allows use of the 'in' operator.
+
+        Examples:
+        >>> s = Struct(x=1)
+        >>> 'x' in s
+        True
+        >>> 'y' in s
+        False
+        >>> s[4] = None
+        >>> 4 in s
+        True
+        >>> s.z = None
+        >>> 'z' in s
+        True
+        """
+        return key in self.__dict__
 
     def __iadd__(self,other):
         """S += S2 is a shorthand for S.merge(S2)."""
@@ -235,12 +255,13 @@ class Struct:
         Optionally, one or more key=value pairs can be given at the end for
         direct update."""
 
-        # The funny name __loc_data__ is to prevent a common variable name which
-        # could be a fieled of a Struct to collide with this parameter. The problem
-        # would arise if the function is called with a keyword with this same name
-        # that a user means to add as a Struct field.
+        # The funny name __loc_data__ is to prevent a common variable name
+        # which could be a fieled of a Struct to collide with this
+        # parameter. The problem would arise if the function is called with a
+        # keyword with this same name that a user means to add as a Struct
+        # field.
         newdict = Struct.__make_dict(self,__loc_data__,**kw)
-        for k,v in newdict.items():
+        for k,v in newdict.iteritems():
             self[k] = v
 
     def merge(self,__loc_data__=None,__conflict_solve=None,**kw):
@@ -318,7 +339,8 @@ class Struct:
         if __conflict_solve:
             inv_conflict_solve_user = __conflict_solve.copy()
             for name, func in [('preserve',preserve), ('update',update),
-                               ('add',add), ('add_flip',add_flip), ('add_s',add_s)]:
+                               ('add',add), ('add_flip',add_flip),
+                               ('add_s',add_s)]:
                 if name in inv_conflict_solve_user.keys():
                     inv_conflict_solve_user[func] = inv_conflict_solve_user[name]
                     del inv_conflict_solve_user[name]
@@ -369,14 +391,14 @@ class Struct:
             return ret
 
     def get(self,attr,val=None):
-        """S.get(k[,d]) -> S[k] if S.has_key(k), else d.  d defaults to None."""
+        """S.get(k[,d]) -> S[k] if k in S, else d.  d defaults to None."""
         try:
             return self[attr]
         except KeyError:
             return val
 
     def setdefault(self,attr,val=None):
-        """S.setdefault(k[,d]) -> S.get(k,d), also set S[k]=d if not S.has_key(k)"""
+        """S.setdefault(k[,d]) -> S.get(k,d), also set S[k]=d if k not in S"""
         if not self.has_key(attr):
             self[attr] = val
         return self.get(attr,val)
@@ -384,8 +406,8 @@ class Struct:
     def allow_new_attr(self, allow = True):
         """ Set whether new attributes can be created inside struct
         
-        This can be used to catch typos by verifying that the attribute user tries to 
-        change already exists in this Struct.
+        This can be used to catch typos by verifying that the attribute user
+        tries to change already exists in this Struct.
         """
         self['__allownew'] = allow
         
